@@ -17,6 +17,9 @@ except ImportError:
     from urllib import quote, unquote
 
 
+from .exceptions import InvalidCharset, InvalidDataURI, InvalidMimeType
+
+
 MIMETYPE_REGEX = r'[\w]+\/[\w\-\+\.]+'
 _MIMETYPE_RE = re.compile('^{}$'.format(MIMETYPE_REGEX))
 
@@ -40,11 +43,11 @@ class DataURI(str):
         parts = ['data:']
         if mimetype is not None:
             if not _MIMETYPE_RE.match(mimetype):
-                raise ValueError("Invalid mimetype: %r" % mimetype)
+                raise InvalidMimeType("Invalid mimetype: %r" % mimetype)
             parts.append(mimetype)
         if charset is not None:
             if not _CHARSET_RE.match(charset):
-                raise ValueError("Invalid charset: %r" % charset)
+                raise InvalidCharset("Invalid charset: %r" % charset)
             parts.extend([';charset=', charset])
         if base64:
             parts.append(';base64')
@@ -65,8 +68,9 @@ class DataURI(str):
     @classmethod
     def from_file(cls, filename, charset=None, base64=True):
         mimetype, _ = mimetypes.guess_type(filename, strict=False)
-        with open(filename) as fp:
+        with open(filename, 'rb') as fp:
             data = fp.read()
+
         return cls.make(mimetype, charset, base64, data)
 
     def __new__(cls, *args, **kwargs):
@@ -106,7 +110,7 @@ class DataURI(str):
     @property
     def text(self):
         if self.charset is None:
-            raise ValueError("DataURI has no encoding set.")
+            raise InvalidCharset("DataURI has no encoding set.")
 
         return self.data.decode(self.charset)
 
@@ -114,7 +118,7 @@ class DataURI(str):
     def _parse(self):
         match = _DATA_URI_RE.match(self)
         if not match:
-            raise ValueError("Not a valid data URI: %r" % self)
+            raise InvalidDataURI("Not a valid data URI: %r" % self)
         mimetype = match.group('mimetype') or None
         name = match.group('name') or None
         charset = match.group('charset') or None
@@ -122,11 +126,7 @@ class DataURI(str):
         if match.group('base64'):
             if BYTES:
                 _charset = charset or 'utf-8'
-                if isinstance(match.group('data'), bytes):
-                    _data = match.group('data')
-                else:
-                    _data = bytes(match.group('data'), _charset)
-
+                _data = bytes(match.group('data'), _charset)
                 data = decode64(_data)
             else:
                 data = decode64(match.group('data'))
